"""Constants for the Efergy integration."""
from datetime import timedelta

ATTRIBUTION = "Data provided by Efergy"

CONF_CURRENT_VALUES = "current_values"

DATA_KEY_API = "api"
DEFAULT_NAME = "Efergy"
DOMAIN = "efergy"

MIN_TIME_BETWEEN_UPDATES = timedelta(seconds=30)
