"""Constants for the Roku integration."""
DOMAIN = "roku"

# Attributes
ATTR_CONTENT_ID = "content_id"
ATTR_FORMAT = "format"
ATTR_KEYWORD = "keyword"
ATTR_MEDIA_TYPE = "media_type"

# Default Values
DEFAULT_PORT = 8060

# Services
SERVICE_SEARCH = "search"
