"""Constants for the WattTime integration."""
import logging

DOMAIN = "watttime"

LOGGER = logging.getLogger(__package__)

CONF_BALANCING_AUTHORITY = "balancing_authority"
CONF_BALANCING_AUTHORITY_ABBREV = "balancing_authority_abbreviation"
CONF_SHOW_ON_MAP = "show_on_map"
