"""Constants for the Ridwell integration."""
import logging

DOMAIN = "ridwell"

LOGGER = logging.getLogger(__package__)

DATA_ACCOUNT = "account"
DATA_COORDINATOR = "coordinator"

SENSOR_TYPE_NEXT_PICKUP = "next_pickup"
