"""Automatically generated by hassfest.

To update, run python3 -m script.hassfest
"""

# fmt: off

USB = [
    {
        "domain": "modem_callerid",
        "vid": "0572",
        "pid": "1340"
    },
    {
        "domain": "velbus",
        "vid": "10CF",
        "pid": "0B1B"
    },
    {
        "domain": "velbus",
        "vid": "10CF",
        "pid": "0516"
    },
    {
        "domain": "velbus",
        "vid": "10CF",
        "pid": "0517"
    },
    {
        "domain": "velbus",
        "vid": "10CF",
        "pid": "0518"
    },
    {
        "domain": "zha",
        "vid": "10C4",
        "pid": "EA60",
        "description": "*2652*"
    },
    {
        "domain": "zha",
        "vid": "10C4",
        "pid": "EA60",
        "description": "*sonoff*plus*"
    },
    {
        "domain": "zha",
        "vid": "10C4",
        "pid": "EA60",
        "description": "*tubeszb*"
    },
    {
        "domain": "zha",
        "vid": "1A86",
        "pid": "7523",
        "description": "*tubeszb*"
    },
    {
        "domain": "zha",
        "vid": "1A86",
        "pid": "7523",
        "description": "*zigstar*"
    },
    {
        "domain": "zha",
        "vid": "1CF1",
        "pid": "0030",
        "description": "*conbee*"
    },
    {
        "domain": "zha",
        "vid": "10C4",
        "pid": "8A2A",
        "description": "*zigbee*"
    },
    {
        "domain": "zha",
        "vid": "10C4",
        "pid": "8B34",
        "description": "*bv 2010/10*"
    },
    {
        "domain": "zwave_js",
        "vid": "0658",
        "pid": "0200"
    },
    {
        "domain": "zwave_js",
        "vid": "10C4",
        "pid": "8A2A",
        "description": "*z-wave*"
    },
    {
        "domain": "zwave_js",
        "vid": "10C4",
        "pid": "EA60"
    }
]
