"""Constants for the Belkin Wemo component."""
DOMAIN = "wemo"

SERVICE_SET_HUMIDITY = "set_humidity"
SERVICE_RESET_FILTER_LIFE = "reset_filter_life"

WEMO_SUBSCRIPTION_EVENT = f"{DOMAIN}_subscription_event"
