"""Define constants for the Sensor.Community integration."""
from datetime import timedelta

ATTR_SENSOR_ID = "sensor_id"

CONF_SENSOR_ID = "sensor_id"

DEFAULT_SCAN_INTERVAL = timedelta(minutes=10)

DOMAIN = "luftdaten"
