"""Constants used by the Ambiclimate component."""

DOMAIN = "ambiclimate"

ATTR_VALUE = "value"

SERVICE_COMFORT_FEEDBACK = "send_comfort_feedback"
SERVICE_COMFORT_MODE = "set_comfort_mode"
SERVICE_TEMPERATURE_MODE = "set_temperature_mode"

STORAGE_KEY = "ambiclimate_auth"
STORAGE_VERSION = 1

AUTH_CALLBACK_NAME = "api:ambiclimate"
AUTH_CALLBACK_PATH = "/api/ambiclimate"
